const UNDEFINED_OPTION = '';

let gTranslator = {};
let gPropertyTypeList = {};
let gSelectedPropertyTypeIndex = 0;
let gSelectedLevelName = UNDEFINED_OPTION;
let gSelectedCategoryName = UNDEFINED_OPTION;

/*
Entry point for every desite form, initialize your data and GUI elements here.
*/
addEventListener('desiteload', async function(event)
{    
    try
    {
        // Translate form's GUI based on application language.    
        gTranslator = await createTranslator('/desite.modules/dictionary.json');
        let appLanguage = await desiteAPI.getApplicationLanguage();    
        setCurrentLanguage(gTranslator, appLanguage);
    }
    catch
    {
        console.log("Module translations are not available.");        
    }
    // Connect 'onSelectionChanged' to the 'selectionChanged' signal. 'onSelectionChanged' will now be continously called on every selection change.
    await desiteAPI.selectionChanged.connect(onSelectionChanged);
    // Create a list of Revit based property types.
    gPropertyTypeList = await desiteAPI.getPropertyTypeList('geometry', 'cpRevit*', true);
    // Create Dropdown menus for Types, Levels and Categories.
    createPropertyTypeDropdownMenu();    
    await createLevelDropdownMenu();
    await createCategoryDropdownMenu();        
    // Update info GUI for currently selected element.
    await updateSelectedElementInfo();
    // Reset displayed geometry based on default filters. 
    await showElementsBasedOnFilter();
    // Create a printable list for up to 12 entries.
    let maxListEntries = 12;
    createPrintableListTable(maxListEntries);
});

/*
Creates dropdown menu options for listed property types.
*/
function createPropertyTypeDropdownMenu()
{
    let dropdown = document.getElementById('dropdownPropertyType');
    dropdown.options.length = 0;
    for (i in gPropertyTypeList)
    {
        let propertyType = gPropertyTypeList[i];
        let option = document.createElement('option');
        option.textContent = propertyType.DisplayName;
        option.value = propertyType.DisplayName;
        dropdown.appendChild(option);
    }    
}

/*
Creates dropdown menu options for available categories in the model, sorted in ascending order
*/
async function createCategoryDropdownMenu()
{
    let undefinedOption = [UNDEFINED_OPTION];
    let categoryNameList = await desiteAPI.getPropertyValues('cpRevitCategory', 'xs:string', 0 ,'geometry', false);
    categoryNameList = categoryNameList.concat(undefinedOption);
    let sortedCategoryNameList = categoryNameList.sort();
    let dropdown = document.getElementById('dropdownCategory');
    for (i in sortedCategoryNameList)
    {
        let categoryName = sortedCategoryNameList[i];
        let option = document.createElement('option');
        option.textContent = categoryName;
        option.value =  categoryName;
        dropdown.appendChild(option);
    }
}

/*
Creates dropdown menu options for available levels in the model, sorted in ascending order.
*/
async function createLevelDropdownMenu()
{            
    let undefinedOption = [UNDEFINED_OPTION];
    let levelNameList = await desiteAPI.getPropertyValues('cpRevitLevel', 'xs:string', 0, 'geometry', false);
    levelNameList = levelNameList.concat(undefinedOption);
    let sortedLevelNameList = levelNameList.sort();
    let dropdown = document.getElementById('dropdownLevel');
    for (i in sortedLevelNameList)
    {
        let levelName = sortedLevelNameList[i];
        let option = document.createElement('option');
        option.textContent = levelName;
        option.value = levelName;
        dropdown.appendChild(option);
    }
}

/*
Updates the GUI with data from the currently selected geometry element.
Currently only one element is supported.
*/
async function updateSelectedElementInfo()
{
    let name = '';
    let propertyValue = '';
    let selectedElements = await desiteAPI.getSelectedElements();
    if (selectedElements.length == 0)
    {
        $("#message").show();
        $("#tableSelectionAndProperties").hide();
    }
    else
    {
        $("#message").hide();
        $("#tableSelectionAndProperties").show();
        if (selectedElements.length == 1)
        {
            let firstSelectedElement = selectedElements[0];
            let propertyName = gPropertyTypeList[gSelectedPropertyTypeIndex].DisplayName;
            let propertyDataType = gPropertyTypeList[gSelectedPropertyTypeIndex].DataType;
            name = await desiteAPI.getPropertyValue(firstSelectedElement, 'cpName', 'xs:string');
            propertyValue = await desiteAPI.getPropertyValue(firstSelectedElement, propertyName, propertyDataType);
        }
        else
        {
            let numberOfSelectedElementsStr = 'Number of selected elements';
            let infoStr = 'Currently only a single element is supported for getting property values.';
            try
            {
                numberOfSelectedElementsStr = getTranslatedString(gTranslator, 'numberOfSelectedElements', numberOfSelectedElementsStr);
                infoStr = getTranslatedString(gTranslator, 'singleElementIsSupported', infoStr);    
            }
            catch
            {
                console.log("Module translations are not available.");
            }
            console.log(`${numberOfSelectedElementsStr}: ${selectedElements.length} - ${infoStr}`);
        }
    }
    $('#inputName').val(name);
    $('#inputPropertyValue').val(propertyValue);
}

/*
Creates a table that can be printed to pdf via button click.
It takes the maximum number of listed elements as a parameter.
*/
async function createPrintableListTable(maxEntries)
{
    let elementList = await desiteAPI.getAllElements();
    $('#tablePrintableList').empty();
    if (elementList.length > 0)
    {
        let rowIndex = 0;
        let bgColors = ["#f7f6f6", "#ffffff"];
        let innerHtml = 
        `<tr style="background: ${bgColors[(rowIndex++)%2]}">;
        <td width="40%">Name</td>;
        <td width="30%">Level</td>;
        <td width="30%">Category</td>;
        </tr>`;
        for (i in elementList)
        {
            if(i == maxEntries)
                break;
            let element = elementList[i];
            let name = await desiteAPI.getPropertyValue(element, 'cpName', 'xs:string');
            let level = await desiteAPI.getPropertyValue(element, 'cpRevitLevel', 'xs:string');
            let category = await desiteAPI.getPropertyValue(element, 'cpRevitCategory', 'xs:string');
            innerHtml += 
            `<tr style="background: ${bgColors[(rowIndex++)%2]}"><td>${name}</td>
            <td>${(level ? level : 'n/a')}</td>
            <td>${(category ? category : 'n/a')}</td>
            </tr>`;         
        }
        $('#tablePrintableList').append(innerHtml);
    }
}

/*
Displays geometric elements based on currently selected filter options.
Geometry can be filtered by 'Category' and/or 'Level'.
*/
async function showElementsBasedOnFilter()
{        
    let elementList = await desiteAPI.getAllElements('geometry');
    if (gSelectedLevelName != UNDEFINED_OPTION)
    {
        let levelNameFilter = `"${gSelectedLevelName}"`;
        elementList = await desiteAPI.filterByProperty(elementList, 'cpRevitLevel', 'xs:string', levelNameFilter, true ,'geometry');
    }    
    if (gSelectedCategoryName != UNDEFINED_OPTION)
    {        
        let categoryNameFilter = `"${gSelectedCategoryName}"`;    
        elementList = await desiteAPI.filterByProperty(elementList, 'cpRevitCategory', 'xs:string', categoryNameFilter, true ,'geometry');
    }

    await desiteAPI.setAllElementsVisible(false, 'geometry')
    await desiteAPI.setElementsVisible(true, elementList);
}

async function onSelectionChanged()
{
    await updateSelectedElementInfo();    
}

async function onZoomToVisibleClicked()
{
   await desiteAPI.zoomToVisible();
}

async function onZoomToSelectedClicked()
{
   await desiteAPI.zoomToSelected();
}

function onDropdownPropertyTypeChanged()
{
    let dropdown = document.getElementById('dropdownPropertyType');
    gSelectedPropertyTypeIndex = dropdown.selectedIndex;
    updateSelectedElementInfo();
}

async function onLevelDropdownMenuChanged() 
{
    let dropdown = document.getElementById('dropdownLevel');
    let selectedIndex = dropdown.selectedIndex;
    gSelectedLevelName = dropdown.options[selectedIndex].value;
    showElementsBasedOnFilter();    
}

async function onCategoryDropdownMenuChanged()
{
    let dropdown = document.getElementById('dropdownCategory');
    let selectedIndex = dropdown.selectedIndex;
    gSelectedCategoryName = dropdown.options[selectedIndex].value;
    showElementsBasedOnFilter();
}

async function onPrintClicked()
{
    await desiteMD.setPageMargins(20,20,20,20);
    await desiteMD.setOrientation(1);   
    await desiteMD.setPaperSize(0);
    let textBlocks =
    {
        content: $('#tablePrintableList').html()
    };
    let filePath = await desiteAPI.getProjectDirectory() + '/desite.modules/firstSteps.module/printTemplate.html';
    await desiteMD.createPdfByTemplate(filePath,'', textBlocks);
}