/*
You can find the Open Layers API documentation here.
https://openlayers.org/en/latest/apidoc/
*/

let gTranslator = {};
let gMap;
let gOverlay;
let gProjection;
let gPropectionType;
let gSelectControl;
let gDefaultLocation = { Latitude: 50, Longitude: 5 };
let gDefaultZoom = 5;
let gShowBuilding = true;
let gShowDocuments = true;
let gFeatures = [];
let gBuildingFeature;
let gDocumentsFeatures = [];

addEventListener('desiteload', async function(event)
{
    try
    {          
        gTranslator = await createTranslator('/desite.modules/dictionary.json');
        let appLanguage = await desiteAPI.getApplicationLanguage();    
        setCurrentLanguage(gTranslator, appLanguage);
    }
    catch
    {
        console.log("Module translations are not available.");
    }
    createMap();    
});

async function updateLocationsOnMap()
{
    for (i in gFeatures)
    {
        let feature = gFeatures[i];
        if (feature.popup)
            destroyPopup(feature);
    }
    gOverlay.removeAllFeatures();

    let visibleFeatures = [];
    if (gShowBuilding)
        visibleFeatures.push(gBuildingFeature); 
    if (gShowDocuments)
        visibleFeatures = visibleFeatures.concat(gDocumentsFeatures);        
    gOverlay.addFeatures(visibleFeatures);
}

async function createMap()
{
    gMap = new OpenLayers.Map("mapdiv");
    gMap.addLayer(new OpenLayers.Layer.OSM());    
    gOverlay = new OpenLayers.Layer.Vector("Overlay");
    gMap.addLayer(gOverlay);
    featureSelect =
    {
        onSelect: createPopup,
        onUnselect: destroyPopup
    };
    gSelectControl =
    {
        selector: new OpenLayers.Control.SelectFeature(gOverlay, featureSelect)
    };
    gMap.addControl(gSelectControl['selector']);
    gSelectControl['selector'].activate();

    gPropectionType =  new OpenLayers.Projection("EPSG:4326"); // WGS 1984 projection
    gProjection = gMap.getProjectionObject(); // The map projection (Spherical Mercator)
    
    let defaultLocation = new OpenLayers.LonLat(gDefaultLocation.Longitude, gDefaultLocation.Latitude).transform(gPropectionType, gProjection);
    gMap.setCenter(defaultLocation, gDefaultZoom);

    let buildingLocation = await desiteAPI.getLocation();
    let buildingLocationGPS = (await desiteAPI.convertCoordinates({ position: { x: buildingLocation.easting, y: buildingLocation.northing, z: 0.0 }, from: "Global", to: "GPS" })).position;

    gBuildingFeature = new OpenLayers.Feature.Vector(
        new OpenLayers.Geometry.Point(buildingLocationGPS.longitude, buildingLocationGPS.latitude).transform(gPropectionType, gProjection),
        {
            description: ("Lat: " + buildingLocationGPS.latitude.toFixed(5) + ", Lon: "+ buildingLocationGPS.longitude.toFixed(5))
        },
        {
            externalGraphic: 'img/home_32x32.png',
            graphicHeight: 32,
            graphicWidth: 32,
            graphicXOffset:-12,
            graphicYOffset:-25
        }
        );   
    gBuildingFeature.attributes.thumbnail = '<img src="img/waldstrasse.png" style="width:128px;height:128px;"/>';
    gFeatures.push(gBuildingFeature);

    let documents = await desiteAPI.getAllElements('documents');
    for (i in documents)
    {
        let doc = documents[i];
        let docLocationStr = await desiteAPI.getPropertyValue(doc,'GpsLocationJSON','xs:string');
        if(!docLocationStr)
            continue;
        let docLocationGPS = JSON.parse(docLocationStr);
        let url = await desiteAPI.getPropertyValue(doc,'GlobalURL','xs:anyURI');
        let feature = new OpenLayers.Feature.Vector(
            new OpenLayers.Geometry.Point(docLocationGPS["Longitude"], docLocationGPS["Latitude"]).transform(gPropectionType, gProjection),
            {
                description: ("Lat: " + buildingLocationGPS.latitude.toFixed(5) + ", Lon: "+ buildingLocationGPS.longitude.toFixed(5))
            },
            {
                externalGraphic: 'img/pin.png',
                graphicHeight: 32,
                graphicWidth: 32,
                graphicXOffset:-12,
                graphicYOffset:-25
            }
            );
        feature.attributes.thumbnail = '<img src="'+ url +'" style="width:128px;height:128px;"/>';
        gDocumentsFeatures.push(feature);
        gFeatures.push(feature);
    } 

    updateLocationsOnMap();
}

function createPopup(feature)
{
    let content = '<div class="markerContent">' + feature.attributes.thumbnail + '<br>' + feature.attributes.description + '</div>';
    feature.popup = new OpenLayers.Popup.FramedCloud(
        "xxxId",
        feature.geometry.getBounds().getCenterLonLat(),
        null,
        content,
        null,
        true,
        function() { gSelectControl['selector'].unselectAll(); }
        );
    gMap.addPopup(feature.popup);
}

function destroyPopup(feature)
{
    feature.popup.destroy();
    feature.popup = null;
}

function onResetLocationClicked()
{
    let defaultLocation = new OpenLayers.LonLat(gDefaultLocation.Longitude, gDefaultLocation.Latitude).transform(gPropectionType, gProjection);
    gMap.setCenter(defaultLocation, gDefaultZoom);
}

function onShowLocationClicked()
{
    if (!gShowBuilding && !gShowDocuments)
        return;

    let extent = gOverlay.getDataExtent();
    gMap.zoomToExtent(extent);
}

function onShowBuildingClicked()
{
    gShowBuilding = $("#showBuildingCheckbox").is(":checked");
    updateLocationsOnMap();
}

function onShowDocumentsClicked()
{
    gShowDocuments = $("#showDocumentsCheckbox").is(":checked");
    updateLocationsOnMap();
}
